# Edit this configuration file to define what should be installed on
# your system.  Help is available in the configuration.nix(5) man page
# and in the NixOS manual (accessible by running ‘nixos-help’).

{ config, pkgs, kaokao, tree-sitter-typst, ... }:

{
  imports =
    [ # Include the results of the hardware scan.
      ./hardware-configuration.nix
    ];

  nixpkgs.config.permittedInsecurePackages = [
                "python-2.7.18.6"
                # "electron-22.3.27"
                "electron-25.9.0"
  #              "electron-16.2.3"
             ];
  
  networking.hostName = "queer-utopia";

  programs.dconf.enable = true;
  services.gnome.evolution-data-server.enable = true;
              
  # Bootloader.
  boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;
  #boot.loader.systemd-boot.configurationLimit = 42;
  #boot.loader.efi.efiSysMountPoint = "/boot/efi";
  boot.kernelPackages = pkgs.linuxPackages_latest;
  
  environment.sessionVariables = {
  DOTNET_ROOT = "${pkgs.dotnet-sdk}";
  };
  
  services.flatpak.enable = true;
  xdg.portal = {
    enable = true;
  };
    
  services.udev.enable = true;
  services.udev.packages = [pkgs.arduino];
  # networking.wireless.enable = true;  # Enables wireless support via wpa_supplicant.
  
  # enables flakes
  nix.settings.experimental-features = ["nix-command" "flakes"];
  # Configure network proxy if necessary
  # networking.proxy.default = "http://user:password@proxy:port/";
  # networking.proxy.noProxy = "127.0.0.1,localhost,internal.domain";

  # Enable networking
  networking.networkmanager.enable = true;

  services.devmon.enable = true;
  services.gvfs.enable = true;
  services.tumbler.enable = true;
  services.udisks2.enable = true;

  # Set your time zone.
  time.timeZone = "Europe/Oslo";

  # Select internationalisation properties.
  i18n.defaultLocale = "en_US.utf8";

  i18n.extraLocaleSettings = {
    LC_ADDRESS = "nb_NO.utf8";
    LC_IDENTIFICATION = "nb_NO.utf8";
    LC_MEASUREMENT = "nb_NO.utf8";
    LC_MONETARY = "nb_NO.utf8";
    LC_NAME = "nb_NO.utf8";
    LC_NUMERIC = "nb_NO.utf8";
    LC_PAPER = "nb_NO.utf8";
    LC_TELEPHONE = "nb_NO.utf8";
    LC_TIME = "nb_NO.utf8";
  };

  # enable wireguard
  networking.firewall.enable = true;
  networking.firewall = {
    # if packets are still dropped, they will show up in dmesg
    logReversePathDrops = true;
    # wireguard trips rpfilter up
    extraCommands = ''
      ip46tables -t mangle -I nixos-fw-rpfilter -p udp -m udp --sport 51820 -j RETURN
      ip46tables -t mangle -I nixos-fw-rpfilter -p udp -m udp --dport 51820 -j RETURN
    '';
    extraStopCommands = ''
      ip46tables -t mangle -D nixos-fw-rpfilter -p udp -m udp --sport 51820 -j RETURN || true
      ip46tables -t mangle -D nixos-fw-rpfilter -p udp -m udp --dport 51820 -j RETURN || true
    '';
  };


  # Enable the i3 and keyboard
  console.keyMap = "dvorak-programmer";
  services.xserver = {
    enable = true;
    # i3
    #displayManager.defaultSession = "";
    #desktopManager = {
    #  xterm.enable = false;
    #  xfce = {
    #    enable = true;
    #    noDesktop = true;
    #    enableXfwm = false;
    #  };
    #};
    desktopManager = {
      plasma5.enable = true;
      #xterm.enable = false;
    };
    displayManager = {
      sddm.enable = true;
      defaultSession = "plasmawayland";
    #    startx.enable = true;
    };
    
     #windowManager.i3 = {
     #   enable = true;
    #};

  };

  services.picom.enable = true;

  #services.xserver.wacom.enable = true;
  
  #xdg.mime.defaultApplications = {
  #"text/html" = "org.qutebrowser.qutebrowser.desktop";
  #"x-scheme-handler/http" = "org.qutebrowser.qutebrowser.desktop";
  #"x-scheme-handler/https" = "org.qutebrowser.qutebrowser.desktop";
  #"x-scheme-handler/about" = "org.qutebrowser.qutebrowser.desktop";
  #"x-scheme-handler/unknown" = "org.qutebrowser.qutebrowser.desktop";
  #};
 # enable the fish
 #users.users.lotte.shell = pkgs.fish;
 users.defaultUserShell = pkgs.fish;
 programs.fish.enable = true;

  # enable the steam
  programs.steam = {
    enable = true;
    remotePlay.openFirewall = true;
    dedicatedServer.openFirewall = true;
  };

  programs.streamdeck-ui = {
    enable = true;
    autoStart = true;
  };

  environment.sessionVariables = rec {
    STEAM_EXTRA_COMPAT_TOOLS_PATHS = "\${HOME}/.steam/root/compatibilitytools.d";
    PATH = [
      "\${HOME}/.local/bin"
    ];
  };

  services.mullvad-vpn.enable = true;

  #enable the joycond
  services.joycond.enable = true;

  #enable bluetooth
  hardware.bluetooth.enable = true;
  services.blueman.enable = true;

  # Configure keymap in X11
  services.xserver = {
    layout = "us";
    xkbVariant = "dvp";
    xkbOptions = "compose:102,numpad:shift3,kpdl:semi,keypad:atm,caps:shift,lv3:ralt_switch";
  };

  # Enable CUPS to print documents.
  services.printing.enable = true;

  virtualisation.docker.enable = true;
  virtualisation.docker.rootless = {
    enable = true;
    setSocketVariable = true;
  };

  # Enable sound with pipewire.
  sound.enable = true;
  hardware.pulseaudio.enable = false;
  security.rtkit.enable = true;
  services.pipewire = {
    enable = true;
    alsa.enable = true;
    alsa.support32Bit = true;
    pulse.enable = true;
    # If you want to use JACK applications, uncomment this
    #jack.enable = true;

    # use the example session manager (no others are packaged yet so this is enabled by default,
    # no need to redefine it in your config for now)
    #media-session.enable = true;
  };

  # Enable touchpad support (enabled default in most desktopManager).
   services.xserver.libinput.enable = true;

  # Define a user account. Don't forget to set a password with ‘passwd’.
  users.users.lotte = {
    isNormalUser = true;
    description = "lotte";
    extraGroups = [ "networkmanager" "wheel" "dialout" "storage" "docker" ];
    packages = with pkgs; [
      firefox
    #  thunderbird
    ];
  };

  services.gnome.at-spi2-core.enable = true;
  
  # Allow unfree packages
  nixpkgs.config.allowUnfree = true;

  # List packages installed in system profile. To search, run:
  # $ nix search wget
  environment.systemPackages = with pkgs; [
    # terminal
    alacritty	
    fish
    any-nix-shell
    
    # window manager & theming stuff	
    #i3
    #sway
    #autotiling
    rofi
    #i3status-rust
    #lxappearance
    #feh
    kaokao.packages.${pkgs.system}.default
    #tree-sitter-typst
    libsForQt5.bismuth

    # streaming / gamign
    parsec-bin
    steam
    lutris-unwrapped
    joycond
    wineWowPackages.stable
    winetricks
    flitter
    duckstation
    minetest
    airshipper
    prismlauncher
    freesweep
    libsForQt5.ksudoku
    libsForQt5.kblocks
    unixtools.quota
    deckmaster
	
    # management
    xsel
    dunst
    clipman
    xclip
    #xfce.xfce4-power-manager
    #xfce.xfce4-screenshooter
     # file
     du-dust
     # audio
     pavucontrol
     # email
     libsForQt5.kmail
     # calendar
     gnome.gnome-calendar
    
    #progaming
    git
    unstable.helix
    godot_4
    nodejs
    nodePackages.vls
    nodePackages.volar
    python310
    mold
    (ruby.withPackages (ps: with ps; [ rails ]))
    bundix
    rubyPackages.rails
    sqlite
    gcc
    arduino
      # rust
      rustup
      rocmPackages.llvm.clang
	    rocmPackages.llvm.llvm
      llvmPackages.bintools
      libclang
      #language servers
      rust-analyzer
      rubyPackages.solargraph
      rubocop
      rufo
      lldb #not a langage server but w/e
      nodePackages.vscode-langservers-extracted
      #vscodium + extensions
      vscode
   
    # creative
    gimp
    libsForQt5.kdenlive
    glaxnimate
    aseprite-unfree
    darktable
    krita
    blender
    blockbench-electron
    xf86_input_wacom
    wacomtablet
    fontforge
    inkscape
     # vrchat
     dotnet-sdk
     #dotnet-runtime
    
    # university
    unstable.typst
    pgadmin4
    postgresql
     # java
     jdk
     eclipses.eclipse-java
     # assembly
     nasm
     #gcc
     gdb

    # i am 10 and i want to donwload and play/hear the movies
    mullvad
    mullvad-vpn
    qbittorrent
    yt-dlp
    cmus
    cmusfm
    clementine
    syncplay
    tor-browser-bundle-bin
    flatpak
    mpv
#    xfce.ristretto
    ffmpeg-full
    
    # messageing n shit
    ungoogled-chromium
    discord
    telegram-desktop
    session-desktop
    element-desktop

    #essentials
    syncthing
    keepassxc
    calc
    libwebp
    libnotify
    sox
    coreutils
    bat
    htop
    xorg.xkill

    # other
    neofetch
    gnome.gnome-keyring
    newsboat
    anki-bin
    projectm
    libsForQt5.okular
    picard
    xfce.mousepad
    xdotool
    audacity
    playerctl
    gnome.adwaita-icon-theme
    tela-circle-icon-theme
    papirus-folders
    libsForQt5.kdeplasma-addons
    unstable.obs-cli
    unstable.libsForQt5.neochat
    libsForQt5.tokodon
    libimobiledevice
    ifuse
  ];


  #Fonts
  fonts = {
    packages = with pkgs; [
      mononoki
      monocraft
      noto-fonts
      noto-fonts-cjk
      noto-fonts-emoji
      arkpandora_ttf
      atkinson-hyperlegible
      andika
    ];
    
    fontconfig = {
      defaultFonts = {
        serif = [ "mononoki" ];
	sansSerif = [ "mononoki" ];
	monospace = [ "mononoki" ];
      };
    };
  };

  
  # bvun
    environment.variables.EDITOR = "hx";

    programs.geary.enable = true;
    services.gnome.gnome-keyring.enable = true;

  # matlab
  nixpkgs.overlays = let
    nix-matlab = import (builtins.fetchTarball "https://gitlab.com/doronbehar/nix-matlab/-/archive/master/nix-matlab-master.tar.gz");
  in [
    nix-matlab.overlay
  ];

  # Some programs need SUID wrappers, can be configured further or are
  # started in user sessions.
  # programs.mtr.enable = true;
  # programs.gnupg.agent = {
  #   enable = true;
  #   enableSSHSupport = true;
  # };

  # List services that you want to enable:

  # Enable the OpenSSH daemon.
  # services.openssh.enable = true;

  # Open ports in the firewall.
  # networking.firewall.allowedTCPPorts = [ ... ];
  # networking.firewall.allowedUDPPorts = [ ... ];
  # Or disable the firewall altogether.
  # networking.firewall.enable = false;

  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leave
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
#  system.stateVersion = "22.05"; # Did you read the comment?

}
